package io.scanbot.sdk.reactnative

import android.app.Application
import com.facebook.react.bridge.Promise
import com.facebook.react.bridge.ReactApplicationContext
import com.facebook.react.bridge.ReactContextBaseJavaModule
import com.facebook.react.bridge.ReactMethod
import com.facebook.react.bridge.ReadableArray
import com.facebook.react.bridge.ReadableMap
import io.scanbot.sdk.reactnative.extensions.toJSON
import io.scanbot.sdk.reactnative.extensions.toJSONArray
import io.scanbot.sdk_wrapper.document.operations.barcode_detection.SBDBarcodesDetection
import io.scanbot.sdk_wrapper.document.operations.doc_analyzer.SBDDocumentQualityAnalyzer
import io.scanbot.sdk_wrapper.document.operations.doc_detection.SBDDocumentDetection
import io.scanbot.sdk_wrapper.document.operations.document.SBDDocument
import io.scanbot.sdk_wrapper.document.operations.image_filters.SBDImageFilters
import io.scanbot.sdk_wrapper.document.operations.image_operations.SBDImageOperations
import io.scanbot.sdk_wrapper.document.operations.legacy_page.SBDLegacyPage
import io.scanbot.sdk_wrapper.document.operations.license.SBDLicenseOperations
import io.scanbot.sdk_wrapper.document.operations.ocr.SBDOcr
import io.scanbot.sdk_wrapper.document.operations.pdf.SBDPdf
import io.scanbot.sdk_wrapper.document.operations.pdf_extraction.SBDExtractFromPdf
import io.scanbot.sdk_wrapper.document.operations.recognizers.check.SBDCheckRecognizer
import io.scanbot.sdk_wrapper.document.operations.recognizers.credit_card.SBDCreditCardRecognizer
import io.scanbot.sdk_wrapper.document.operations.recognizers.doc_data_extractor.SBDDocDataExtractorRecognizer
import io.scanbot.sdk_wrapper.document.operations.recognizers.ehic.SBDEhicRecognizer
import io.scanbot.sdk_wrapper.document.operations.recognizers.medical_cert.SBDMedicalCertRecognizer
import io.scanbot.sdk_wrapper.document.operations.recognizers.mrz.SBDMrzRecognizer
import io.scanbot.sdk_wrapper.document.operations.setup.SBDSetupOperations
import io.scanbot.sdk_wrapper.document.operations.tif.SBDTiff
import org.json.JSONObject
import kotlin.math.roundToInt


class ScanbotSDKModule(reactContext: ReactApplicationContext) :
    ReactContextBaseJavaModule(reactContext) {

    companion object {
        const val NAME = "RNScanbotSDK"
    }

    override fun getName(): String {
        return NAME
    }

    @ReactMethod
    fun initializeSDK(options: ReadableMap, promise: Promise) {
        reactApplicationContextIfActiveOrWarn?.let {
            SBDSetupOperations.initializeSdk(
                it.applicationContext as Application,
                options.toJSON(),
                ScanbotSDKPluginResultDelegate(promise)
            )
        } ?: run {
            promise.reject("", "React Native Instance has already disappeared.")
        }
    }

    @ReactMethod
    fun getLicenseInfo(promise: Promise) {
        SBDLicenseOperations.getLicenseInfo(
            ScanbotSDKPluginResultDelegate(
                promise
            )
        )
    }

    @ReactMethod
    fun detectDocument(imageFileUri: String, promise: Promise) {
        SBDDocumentDetection.detectDocumentOnImage(
            imageFileUri, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun documentQualityAnalyzer(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val configuration = params.getMap("configuration")!!.toJSON()

        SBDDocumentQualityAnalyzer.analyze(
            imageFileUri, configuration, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @Suppress("UNCHECKED_CAST")
    @ReactMethod
    fun applyImageFilters(imageFileUri: String, filters: ReadableArray, promise: Promise) {
        val filterAsListOfJsons = filters.toArrayList().map { JSONObject(it as Map<String, Any>) }

        SBDImageFilters.applyFiltersOnImage(
            imageFileUri, filterAsListOfJsons, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun getOCRConfigs(promise: Promise) {
        SBDOcr.getConfigs(ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun performOCR(args: ReadableMap, promise: Promise) {
        SBDOcr.perform(args.toJSON(), ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun createPDF(args: ReadableMap, promise: Promise) {
        SBDPdf.create(args.toJSON(), ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun writeTIFF(args: ReadableMap, promise: Promise) {
        SBDTiff.write(args.toJSON(), ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun extractPagesFromPdf(options: ReadableMap, promise: Promise) {
        SBDExtractFromPdf.extractLegacyPages(
            options.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun extractImagesFromPdf(options: ReadableMap, promise: Promise) {
        SBDExtractFromPdf.extractImages(
            options.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun rotateImage(imageFileUri: String, degrees: Double, promise: Promise) {
        SBDImageOperations.rotateImage(
            imageFileUri, degrees, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun createPage(imageUri: String, promise: Promise) {
        SBDLegacyPage.createPage(imageUri, ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun detectDocumentOnPage(pageMap: ReadableMap, promise: Promise) {
        SBDDocumentDetection.detectDocumentOnPage(
            pageMap.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun detectBarcodesOnImage(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val barcodeScannerConfiguration = params.getMap("configuration")!!.toJSON()

        SBDBarcodesDetection.detectOnImage(
            imageFileUri, barcodeScannerConfiguration, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @Suppress("UNCHECKED_CAST")
    @ReactMethod
    fun applyImageFiltersOnPage(pageMap: ReadableMap, filters: ReadableArray, promise: Promise) {
        val filterAsListOfJsons = filters.toArrayList().map { JSONObject(it as Map<String, Any>) }

        SBDImageFilters.applyFiltersOnPage(
            pageMap.toJSON(), filterAsListOfJsons, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun rotatePage(pageMap: ReadableMap, times: Double, promise: Promise) {
        SBDLegacyPage.rotatePage(
            pageMap.toJSON(), times.roundToInt(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun removePage(pageMap: ReadableMap, promise: Promise) {
        SBDLegacyPage.removePage(
            pageMap.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun setDocumentImage(pageMap: ReadableMap, imageUri: String, promise: Promise) {
        SBDLegacyPage.setDocumentImage(
            pageMap.toJSON(), imageUri, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun cleanup(promise: Promise) {
        SBDSetupOperations.cleanup(ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun recognizeMrz(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val configuration = params.getMap("configuration")!!.toJSON()

        SBDMrzRecognizer.recognize(
            imageFileUri, configuration, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun recognizeCheck(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val configuration = params.getMap("configuration")!!.toJSON()
        val acceptedCheckStandards = params.getArray("acceptedCheckStandards")?.toJSONArray()

        SBDCheckRecognizer.recognize(
            imageFileUri,
            configuration,
            acceptedCheckStandards,
            ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun recognizeMedicalCertificate(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val configuration = params.getMap("configuration")!!.toJSON()

        SBDMedicalCertRecognizer.recognize(
            imageFileUri, configuration, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun recognizeEHIC(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val configuration = params.getMap("configuration")!!.toJSON()

        SBDEhicRecognizer.recognize(
            imageFileUri, configuration, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun recognizeCreditCard(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val configuration = params.getMap("configuration")!!.toJSON()

        SBDCreditCardRecognizer.recognize(
            imageFileUri, configuration, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun documentDataExtractor(params: ReadableMap, promise: Promise) {
        val imageFileUri = params.getString("imageFileUri")!!
        val configuration = params.getMap("configuration")!!.toJSON()

        SBDDocDataExtractorRecognizer.recognize(
            imageFileUri, configuration, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun refreshImageUris(map: ReadableMap, promise: Promise) {
        SBDLegacyPage.refreshImageUris(
            map.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun getImageData(imageFileUri: String, promise: Promise) {
        SBDImageOperations.getImageData(
            imageFileUri, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    //region DocumentAPIs

    @ReactMethod
    fun createDocument(params: ReadableMap, promise: Promise) {
        SBDDocument.create(
            params.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun createDocumentFromLegacyPages(params: ReadableMap, promise: Promise) {
        SBDDocument.createFromLegacyPages(
            params.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun createDocumentFromPDF(pdfUri: String, promise: Promise) {
        SBDDocument.createFromPDF(
            pdfUri, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun documentExists(documentID: String, promise: Promise) {
        SBDDocument.exists(documentID, ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun loadDocument(documentID: String, promise: Promise) {
        SBDDocument.load(documentID, ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun storedDocumentIDs(promise: Promise) {
        SBDDocument.storedIDs(ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun cloneDocument(documentID: String, promise: Promise) {
        SBDDocument.clone(documentID, ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun deleteDocument(documentID: String, promise: Promise) {
        SBDDocument.delete(documentID, ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun deleteAllDocuments(promise: Promise) {
        SBDDocument.deleteAll(ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun createPDFForDocument(params: ReadableMap, promise: Promise) {
        SBDPdf.createFromDocument(
            params.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun createTIFFForDocument(params: ReadableMap, promise: Promise) {
        SBDTiff.createFromDocument(
            params.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun addPage(params: ReadableMap, promise: Promise) {
        SBDDocument.addPage(params.toJSON(), ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun movePage(params: ReadableMap, promise: Promise) {
        SBDDocument.movePage(params.toJSON(), ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun modifyPage(params: ReadableMap, promise: Promise) {
        SBDDocument.modifyPage(
            params.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun removePageFromDocument(params: ReadableMap, promise: Promise) {
        SBDDocument.removePage(
            params.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun removeAllPages(documentID: String, promise: Promise) {
        SBDDocument.removeAllPages(
            documentID, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    //endregion DocumentAPIs

    //region ImageRefs

    @ReactMethod
    fun imageRefDeserialize(uniqueId: String, promise: Promise) {
        SBDImageOperations.imageRefDeserialize(
            uniqueId, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefSerialize(uniqueId: String, promise: Promise) {
        SBDImageOperations.imageRefSerialize(
            uniqueId, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefFromImageFileUri(uri: String, options: ReadableMap, promise: Promise) {
        SBDImageOperations.imageRefFromImageFileUri(
            uri, options.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefFromEncodedBuffer(buffer: String, options: ReadableMap, promise: Promise) {
        SBDImageOperations.imageRefFromEncodedBuffer(
            buffer, options.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefClone(uniqueId: String, promise: Promise) {
        SBDImageOperations.imageRefClone(
            uniqueId, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefHibernate(uniqueId: String, promise: Promise) {
        SBDImageOperations.imageRefHibernate(
            uniqueId, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefClear(uniqueId: String, promise: Promise) {
        SBDImageOperations.imageRefClear(
            uniqueId, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefInfo(uniqueId: String, promise: Promise) {
        SBDImageOperations.imageRefInfo(
            uniqueId, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefSaveImage(uniqueId: String, path: String, options: ReadableMap, promise: Promise) {
        SBDImageOperations.imageRefSaveImage(
            uniqueId, path, options.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefEncodeImage(uniqueId: String, options: ReadableMap, promise: Promise) {
        SBDImageOperations.imageRefEncodeImage(
            uniqueId, options.toJSON(), ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun imageRefRelease(uniqueId: String, promise: Promise) {
        SBDImageOperations.imageRefRelease(
            uniqueId, ScanbotSDKPluginResultDelegate(promise)
        )
    }

    @ReactMethod
    fun makeSnapshot(promise: Promise) {
        SBDImageOperations.makeSnapshot(ScanbotSDKPluginResultDelegate(promise))
    }

    @ReactMethod
    fun imageRefReleaseAll(promise: Promise) {
        SBDImageOperations.releaseAllImageRefs(ScanbotSDKPluginResultDelegate(promise))
    }

    //endregion ImageRefs
}
