package io.scanbot.sdk.reactnative.components.barcode_scanner_view

import android.Manifest
import android.app.Activity
import android.content.Context
import android.content.pm.PackageManager
import android.content.res.ColorStateList
import android.graphics.Color
import android.util.AttributeSet
import android.view.View
import android.view.ViewGroup
import android.widget.Button
import android.widget.FrameLayout
import androidx.core.app.ActivityCompat
import androidx.core.view.isVisible
import com.facebook.react.bridge.LifecycleEventListener
import com.facebook.react.bridge.ReactContext
import com.facebook.react.uimanager.UIManagerHelper
import io.scanbot.barcodesdk.plugin.reactnative.components.barcode_scanner_view.BarcodeScannerResultEvent
import io.scanbot.barcodesdk.plugin.reactnative.components.barcode_scanner_view.BarcodeSelectionResultEvent
import io.scanbot.sdk.ScanbotSDK
import io.scanbot.sdk.barcode.BarcodeItem
import io.scanbot.sdk.barcode.BarcodeScannerResult
import io.scanbot.sdk.barcode.textWithExtension
import io.scanbot.sdk.barcode.ui.BarcodeOverlayTextFormat
import io.scanbot.sdk.barcode.ui.BarcodePolygonsView
import io.scanbot.sdk.barcode.ui.BarcodeScannerView
import io.scanbot.sdk.barcode.ui.IBarcodeScannerFinderViewController
import io.scanbot.sdk.barcode.ui.IBarcodeScannerSelectionOverlayController
import io.scanbot.sdk.barcode.ui.IBarcodeScannerViewCallback
import io.scanbot.sdk.camera.CaptureInfo
import io.scanbot.sdk.camera.FrameHandlerResult
import io.scanbot.sdk.reactnative.extensions.toPx
import io.scanbot.sdk.reactnative.models.BarcodeItemOverlayViewConfig
import io.scanbot.sdk.reactnative.models.BaseEvent
import io.scanbot.sdk.reactnative.models.CameraConfiguration
import io.scanbot.sdk.reactnative.models.ComponentEvent
import io.scanbot.sdk.reactnative.models.FinderViewConfiguration
import io.scanbot.sdk.reactnative.models.PolygonStyleConfig
import io.scanbot.sdk.reactnative.models.TextStyleConfig
import io.scanbot.sdk.ui.camera.CameraUiSettings
import io.scanbot.sdk_wrapper.barcode.extensions.uuid

class ScanbotBarcodeScannerView(
    context: Context, attrs: AttributeSet?, defStyleAttr: Int, defStyleRes: Int
) : FrameLayout(context, attrs, defStyleAttr, defStyleRes) {

    constructor(context: Context) : this(context, null)
    constructor(context: Context, attrs: AttributeSet?) : this(context, attrs, 0)
    constructor(context: Context, attrs: AttributeSet?, defStyleAttr: Int) : this(
        context, attrs, defStyleAttr, 0
    )

    companion object {
        private const val CAMERA_PERMISSION_REQUEST_CODE = 3331
        private const val INITIAL_FLASH_STATE = false
        private const val BARCODE_SCANNING_INTERVAL = 0L
    }

    val barcodeScanner = ScanbotSDK(context).createBarcodeScanner()

    lateinit var scannerView: BarcodeScannerView
        private set

    var flashState: Boolean = INITIAL_FLASH_STATE
    var polygonStyle = PolygonStyleConfig()
    var textStyle = TextStyleConfig()

    private var cameraPermissionRequested = false

    private var barcodeItemOverlayViewBinders: MutableMap<String, BarcodeItemOverlayViewConfig>? =
        null

    private val barcodeItemOverlayViewBinder: BarcodePolygonsView.BarcodeItemViewBinder =
        object : BarcodePolygonsView.BarcodeItemViewBinder {

            override fun bindView(view: View, barcodeItem: BarcodeItem, shouldHighlight: Boolean) {
                (view as? Button)?.let { buttonView ->
                    barcodeItemOverlayViewBinders?.get(barcodeItem.uuid())
                        ?.let { overlayViewConfig ->

                            if (textStyle.textFormat == BarcodeOverlayTextFormat.NONE) {
                                buttonView.isVisible = false
                                return
                            }

                            var configTextContainerColor =
                                overlayViewConfig.textContainerColorParsed
                            var configTextColor = overlayViewConfig.textColorParsed
                            var configText = overlayViewConfig.text

                            // If caching is disabled use general settings as starting point (not last bind) after refreshRate
                            if (overlayViewConfig.refreshRate > 0 && overlayViewConfig.lastBind > 0 && (System.currentTimeMillis() - overlayViewConfig.lastBind >= overlayViewConfig.refreshRate)) {
                                configTextContainerColor = null
                                configTextColor = null
                                configText = BarcodeItemOverlayViewConfig.LOADING_TEXT
                            }

                            // buttonView doesn't use global overlay configuration by default
                            buttonView.backgroundTintList = ColorStateList.valueOf(
                                configTextContainerColor ?: textStyle.textContainerColor!!
                            )

                            buttonView.compoundDrawables.forEach {
                                it?.setTint(configTextColor ?: textStyle.textColor!!)
                            }
                            buttonView.setTextColor(configTextColor ?: textStyle.textColor!!)

                            buttonView.maxLines = when (textStyle.textFormat) {
                                BarcodeOverlayTextFormat.CODE -> 1
                                else -> 2
                            }

                            buttonView.text = configText ?: run {
                                when (textStyle.textFormat) {
                                    BarcodeOverlayTextFormat.CODE -> barcodeItem.textWithExtension

                                    BarcodeOverlayTextFormat.CODE_AND_TYPE -> "${
                                        barcodeItem.format.name.replace("_", " ")
                                    }\n${barcodeItem.textWithExtension}"

                                    else -> ""
                                }
                            }

                            buttonView.isVisible = true

                        } ?: run {
                        buttonView.isVisible = false
                    }
                }
            }
        }

    private var lifecycleEventListener: LifecycleEventListener = object : LifecycleEventListener {
        override fun onHostResume() {
            requestCameraPermissionIfNotGranted()
            scannerView.viewController.onResume()
        }

        override fun onHostPause() {
            scannerView.viewController.onPause()
        }

        override fun onHostDestroy() {

        }
    }

    init {
        initBarcodeScannerView()

        layoutParams = ViewGroup.LayoutParams(
            ViewGroup.LayoutParams.MATCH_PARENT,
            ViewGroup.LayoutParams.MATCH_PARENT,
        )

        setBackgroundColor(Color.BLACK)

        addView(scannerView)
    }

    private fun initBarcodeScannerView() {
        scannerView = BarcodeScannerView(context).apply {
            initCamera(CameraUiSettings(true))

            initScanningBehavior(barcodeScanner, {
                (it as? FrameHandlerResult.Success)?.value?.let { result ->
                    sendEvent(BarcodeScannerResultEvent(result))

                    addBarcodeItemOverlayViewInitialBinder(result)
                }

                false
            }, object : IBarcodeScannerViewCallback {
                override fun onCameraOpen() {
                    viewController.useFlash(flashState)
                }

                override fun onPictureTaken(image: ByteArray, captureInfo: CaptureInfo) {
                }

                override fun onSelectionOverlayBarcodeClicked(barcodeItem: BarcodeItem) {
                    sendEvent(BarcodeSelectionResultEvent(barcodeItem))
                }
            })

            viewController.barcodeScanningInterval = BARCODE_SCANNING_INTERVAL

            configureFinderViewControllerWithDefaults(finderViewController)

            cameraConfiguration.setPhysicalZoomRange(CameraConfiguration.DEFAULT_ZOOM_RANGE)

            configureSelectionOverlayControllerWithDefaults(selectionOverlayController)

            (context as ReactContext).addLifecycleEventListener(lifecycleEventListener)
        }
    }

    private fun configureFinderViewControllerWithDefaults(finderViewController: IBarcodeScannerFinderViewController) {
        finderViewController.setStrokeColor(FinderViewConfiguration.DEFAULT_STROKE_COLOR)
        finderViewController.setOverlayColor(FinderViewConfiguration.DEFAULT_OVERLAY_COLOR)
        finderViewController.setFinderEnabled(FinderViewConfiguration.DEFAULT_FINDER_ENABLED)
        finderViewController.setStrokeWidth(
            FinderViewConfiguration.DEFAULT_STROKE_WIDTH.toPx(
                resources
            )
        )
        finderViewController.setFinderMinPadding(
            FinderViewConfiguration.DEFAULT_FINDER_MIN_PADDING.toPx(
                resources
            )
        )
        finderViewController.setFinderInset(
            FinderViewConfiguration.DEFAULT_FINDER_INSETS.left.toPx(resources),
            FinderViewConfiguration.DEFAULT_FINDER_INSETS.top.toPx(resources),
            FinderViewConfiguration.DEFAULT_FINDER_INSETS.right.toPx(resources),
            FinderViewConfiguration.DEFAULT_FINDER_INSETS.bottom.toPx(resources)
        )
        finderViewController.setRequiredAspectRatios(FinderViewConfiguration.DEFAULT_ASPECT_RATIO)
    }

    private fun configureSelectionOverlayControllerWithDefaults(selectionOverlayController: IBarcodeScannerSelectionOverlayController) {
        selectionOverlayController.setBarcodeAppearanceDelegate(object :
            BarcodePolygonsView.BarcodeAppearanceDelegate {
            override fun getPolygonStyle(
                defaultStyle: BarcodePolygonsView.BarcodePolygonStyle, barcodeItem: BarcodeItem
            ): BarcodePolygonsView.BarcodePolygonStyle {
                return defaultStyle.copy(
                    fillColor = PolygonStyleConfig.DEFAULT_FILL_COLOR,
                    fillHighlightedColor = PolygonStyleConfig.DEFAULT_FILL_COLOR,
                    strokeColor = PolygonStyleConfig.DEFAULT_STROKE_COLOR,
                    strokeHighlightedColor = PolygonStyleConfig.DEFAULT_STROKE_COLOR
                )
            }

            override fun getTextViewStyle(
                defaultStyle: BarcodePolygonsView.BarcodeTextViewStyle, barcodeItem: BarcodeItem
            ): BarcodePolygonsView.BarcodeTextViewStyle {
                return defaultStyle.copy(
                    textColor = TextStyleConfig.DEFAULT_TEXT_COLOR,
                    textHighlightedColor = TextStyleConfig.DEFAULT_TEXT_COLOR,
                    textContainerColor = TextStyleConfig.DEFAULT_TEXT_CONTAINER_COLOR,
                    textContainerHighlightedColor = TextStyleConfig.DEFAULT_TEXT_CONTAINER_COLOR,
                    textFormat = TextStyleConfig.DEFAULT_TEXT_FORMAT
                )
            }
        })
    }

    override fun onAttachedToWindow() {
        super.onAttachedToWindow()
        this.scannerView.viewController.onResume()
    }

    override fun onDetachedFromWindow() {
        this.scannerView.viewController.onPause()
        super.onDetachedFromWindow()
    }

    fun removeAttachedListeners() {
        (context as ReactContext).removeLifecycleEventListener(lifecycleEventListener)
    }

    override fun requestLayout() {
        super.requestLayout()
        post {
            layoutView()
        }
    }

    private fun View.layoutView() {
        measure(
            MeasureSpec.makeMeasureSpec(measuredWidth, MeasureSpec.EXACTLY),
            MeasureSpec.makeMeasureSpec(measuredHeight, MeasureSpec.EXACTLY)
        )
        layout(left, top, right, bottom)
    }

    private fun sendEvent(event: BaseEvent) {
        val surfaceId = UIManagerHelper.getSurfaceId(this)

        UIManagerHelper.getEventDispatcherForReactTag((context as ReactContext), id)
            ?.dispatchEvent(ComponentEvent(event, surfaceId, id))
    }

    private fun requestCameraPermissionIfNotGranted() {
        val activity: Activity = (context as ReactContext).currentActivity ?: return

        if (!cameraPermissionRequested && ActivityCompat.checkSelfPermission(
                activity, Manifest.permission.CAMERA
            ) != PackageManager.PERMISSION_GRANTED
        ) {
            // shouldShowRequestPermissionRationale can be changed from another place in the app,
            // that's why we are using custom flag
            cameraPermissionRequested = true

            ActivityCompat.requestPermissions(
                activity, arrayOf(Manifest.permission.CAMERA), CAMERA_PERMISSION_REQUEST_CODE
            )
        }
    }

    fun setOverlayAppearance() {
        scannerView.selectionOverlayController.setBarcodeAppearanceDelegate(object :
            BarcodePolygonsView.BarcodeAppearanceDelegate {
            override fun getPolygonStyle(
                defaultStyle: BarcodePolygonsView.BarcodePolygonStyle, barcodeItem: BarcodeItem
            ): BarcodePolygonsView.BarcodePolygonStyle {
                val overlayViewConfig =
                    barcodeItemOverlayViewBinders?.get(barcodeItem.uuid())?.let {

                        // If caching is disabled use general settings as starting point (not last bind) after refreshRate
                        if (it.refreshRate > 0 && it.lastBind > 0 && (System.currentTimeMillis() - it.lastBind >= it.refreshRate)) {
                            null
                        } else {
                            it
                        }
                    }

                return defaultStyle.copy(
                    fillColor = overlayViewConfig?.polygonColorParsed ?: polygonStyle.fillColor!!,
                    fillHighlightedColor = overlayViewConfig?.polygonColorParsed
                        ?: polygonStyle.fillColor!!,
                    strokeColor = overlayViewConfig?.strokeColorParsed
                        ?: polygonStyle.strokeColor!!,
                    strokeHighlightedColor = overlayViewConfig?.strokeColorParsed
                        ?: polygonStyle.strokeColor!!,
                )
            }

            override fun getTextViewStyle(
                defaultStyle: BarcodePolygonsView.BarcodeTextViewStyle, barcodeItem: BarcodeItem
            ): BarcodePolygonsView.BarcodeTextViewStyle {
                return defaultStyle.copy(
                    textColor = textStyle.textColor!!,
                    textHighlightedColor = textStyle.textColor!!,
                    textContainerColor = textStyle.textContainerColor!!,
                    textContainerHighlightedColor = textStyle.textContainerColor!!,
                    textFormat = textStyle.textFormat!!
                )
            }
        })
    }

    private fun addBarcodeItemOverlayViewInitialBinder(barcodeResult: BarcodeScannerResult) {
        barcodeItemOverlayViewBinders?.let { bindersList ->
            barcodeResult.barcodes.forEach { barcodeItem ->
                val barcodeItemUuid = barcodeItem.uuid()

                if (!bindersList.contains(barcodeItemUuid)) {
                    bindersList[barcodeItemUuid] =
                        BarcodeItemOverlayViewConfig(text = BarcodeItemOverlayViewConfig.LOADING_TEXT)
                }
            }
        }
    }

    fun updateBarcodeItemOverlayViewBinder(
        barcodeItemUuid: String, barcodeItemOverlayViewConfig: BarcodeItemOverlayViewConfig
    ) {
        barcodeItemOverlayViewBinders?.let { bindersList ->
            if (bindersList.contains(barcodeItemUuid)) {
                bindersList[barcodeItemUuid] = barcodeItemOverlayViewConfig
            }
        }
    }

    fun setBarcodeItemOverlayViewBinderCallback() {
        if (barcodeItemOverlayViewBinders == null) {
            barcodeItemOverlayViewBinders = mutableMapOf()

            scannerView.selectionOverlayController.setBarcodeItemViewBinder(
                barcodeItemOverlayViewBinder
            )
        }
    }
}
