import { ToJsonConfiguration } from '../utils/json/JsonSerializationTypes';
import { DeepPartial, PartiallyConstructible, Rectangle } from '../utils/utils';
/**
Grammar of pattern matching.

- `WILDCARD`:
   Simple pattern matching where # stands for any digit and ? stands for any character.
   All other characters represent themselves.
- `REGEX`:
   Regular expression pattern matching. Uses ECMAScript grammar.
*/
export type PatternGrammar = 'WILDCARD' | 'REGEX';
export declare const PatternGrammarValues: PatternGrammar[];
/**
Structure containing recognized word text and bounds.
*/
export declare class WordBox extends PartiallyConstructible {
    /**
      Recognized word text.
      */
    readonly text: string;
    /**
      Bounding rectangle of the recognized word.
      */
    readonly boundingRect: Rectangle;
    /**
      Confidence of the recognition.
      
      Default is 0.0
      */
    readonly recognitionConfidence: number;
    /** @param source {@displayType `DeepPartial<WordBox>`} */
    constructor(source?: DeepPartial<WordBox>);
    serialize(config?: ToJsonConfiguration): DeepPartial<WordBox>;
}
/**
Structure containing recognized symbol text and bounds.
*/
export declare class SymbolBox extends PartiallyConstructible {
    /**
      Recognized symbol text.
      */
    readonly symbol: string;
    /**
      Bounding rectangle of the recognized symbol.
      */
    readonly boundingRect: Rectangle;
    /**
      Confidence of the recognition.
      */
    readonly recognitionConfidence: number;
    /** @param source {@displayType `DeepPartial<SymbolBox>`} */
    constructor(source?: DeepPartial<SymbolBox>);
    serialize(config?: ToJsonConfiguration): DeepPartial<SymbolBox>;
}
/**
The result of the text line recognition.
*/
export declare class TextPatternScannerResult extends PartiallyConstructible {
    /**
      Raw recognized string.
      */
    readonly rawText: string;
    /**
      Boxes for each recognized word.
      */
    readonly wordBoxes: WordBox[];
    /**
      Boxes for each recognized symbol.
      */
    readonly symbolBoxes: SymbolBox[];
    /**
      Confidence of the recognition.
      
      Default is 0.0
      */
    readonly confidence: number;
    /**
      Whether the validation was successful.
      
      Default is false
      */
    readonly validationSuccessful: boolean;
    /** @param source {@displayType `DeepPartial<TextPatternScannerResult>`} */
    constructor(source?: DeepPartial<TextPatternScannerResult>);
    serialize(config?: ToJsonConfiguration): DeepPartial<TextPatternScannerResult>;
}
/**
Base class for content validators.
*/
export type ContentValidator = DefaultContentValidator | PatternContentValidator;
/**
Default content validator. Accepts only non-empty strings.
*/
export declare class DefaultContentValidator extends PartiallyConstructible {
    readonly _type: 'DefaultContentValidator';
    /**
      OCR whitelist. Empty string means no restriction.
      
      Default is ""
      */
    allowedCharacters: string;
    /** @param source {@displayType `DeepPartial<DefaultContentValidator>`} */
    constructor(source?: DeepPartial<DefaultContentValidator>);
}
/**
Pattern content validator.
*/
export declare class PatternContentValidator extends PartiallyConstructible {
    readonly _type: 'PatternContentValidator';
    /**
      OCR whitelist. Empty string means no restriction.
      
      Default is ""
      */
    allowedCharacters: string;
    /**
      Pattern to match. It can be a simple pattern or a regular expression.
      */
    pattern: string;
    /**
      Whether the pattern should match the whole string or just a substring.
      
      Default is false
      */
    matchSubstring: boolean;
    /**
      Grammar of pattern matching.
      
      Default is WILDCARD
      */
    patternGrammar: PatternGrammar;
    /** @param source {@displayType `DeepPartial<PatternContentValidator>`} */
    constructor(source?: DeepPartial<PatternContentValidator>);
}
/**
Configuration for the text pattern scanner.
*/
export declare class TextPatternScannerConfiguration extends PartiallyConstructible {
    /**
      Maximum image size (height or width) for OCR process. 0 - do not rescale.
      
      Default is 0
      */
    ocrResolutionLimit: number;
    /**
      Maximum number of accumulated frames to inspect before actual result is returned.
      
      Default is 3
      */
    maximumNumberOfAccumulatedFrames: number;
    /**
      Minimum number of accumulated frames that have equal result.
      
      Default is 2
      */
    minimumNumberOfRequiredFramesWithEqualScanningResult: number;
    /**
      Content validator.
      */
    validator: ContentValidator;
    /** @param source {@displayType `DeepPartial<TextPatternScannerConfiguration>`} */
    constructor(source?: DeepPartial<TextPatternScannerConfiguration>);
}
//# sourceMappingURL=TextPatternScannerTypes.d.ts.map