import { DocumentScannerEngineMode } from '../document_scanner';
import { AspectRatio } from '../utils';
import {
  CameraModule,
  CameraPreviewMode,
  CapturePhotoQualityPrioritization,
  CheckStandard,
  CheckUserGuidanceStrings,
  CroppingAccessibilityConfiguration,
  DialogButtonStyle,
  DialogStyle,
  DocumentDataType,
  DocumentScannerAccessibilityConfiguration,
  DocumentsDisplayConfiguration,
  FieldsDisplayConfiguration,
  FileEncryptionMode,
  FinderDocumentScannerAccessibilityConfiguration,
  HealthInsuranceCardRecognizerParameters,
  ImageFilterType,
  MedicalCertificateUserGuidanceStrings,
  OrientationLockMode,
  Size,
  StorageImageFormat,
} from './types';

export interface CheckScannerScreenConfiguration {
  /** Accepted check standards. Optional, by default - all checks are accepted. With an empty list no filter is applied and all checks are accepted. */
  acceptedCheckStandards?: CheckStandard[];

  /** The preferred camera module (default - BACK) */
  cameraModule?: CameraModule;

  /** Background color outside of the finder window. */
  cameraOverlayColor?: string;

  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** Whether the cancel button is hidden or not. iOS only. */
  cancelButtonHidden?: boolean;

  /** Set high-resolution Check image capturing. If set to TRUE a high-resolution photo of the check is captured to run the recognition on. Upon successful recognition the recognition result also contains a high-resolution image of the check. If set to FALSE, the video frames will be used for recognition and result image delivery. The default value is TRUE. */
  captureHighResolutionImage?: boolean;

  /** Title of the button that opens the screen where the user can allow the usage of the camera by the app. */
  enableCameraButtonTitle?: string;

  /** Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera. */
  enableCameraExplanationText?: string;

  /** The style of the dialog/alert in case of error. iOS Only. */
  errorDialogStyle?: DialogStyle;

  /** The style of the dialogs/alerts OK button in case of error. iOS Only. */
  errorDialogButtonStyle?: DialogButtonStyle;

  /** String being displayed as the title in the error alert that may occur when the recognition fails on captured high-resolution images. iOS only. */
  errorAlertTitle?: string;

  /** String being displayed as the message in the error alert that may occur when the recognition fails on captured high-resolution images. iOS only. */
  errorAlertMessage?: string;

  /** String being displayed as the button text in the error alert that may occur when the recognition fails on captured high-resolution images. iOS only. */
  errorAlertButtonText?: string;

  /** Aspect ratio of finder frame (width \ height), which is used to build actual finder frame. Default is 1 - it is a square frame, which is good for QR capturing. */
  finderAspectRatio?: AspectRatio;

  /** Foreground color of the detection overlay. */
  finderLineColor?: string;

  /** Width of finder frame border. Default is 2. */
  finderLineWidth?: number;

  /** Controls whether the flash toggle button is hidden or not. iOS only. */
  flashButtonHidden?: boolean;

  /** Title of the flash toggle button. iOS only. */
  flashButtonTitle?: string;

  /** Controls whether the flash should be initially enabled. The default value is FALSE. */
  flashEnabled?: boolean;

  /** UI Interface orientation lock mode */
  orientationLockMode?: OrientationLockMode;

  /** The background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** The color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** The color of all inactive toggle buttons in the toolbar. */
  topBarButtonsInactiveColor?: string;

  /** The background color of the user guidance hints. */
  userGuidanceBackgroundColor?: string;

  /** The text color of the user guidance hints. */
  userGuidanceTextColor?: string;

  /** Configuration that helps to override default hint values. */
  userGuidanceStringValues?: CheckUserGuidanceStrings;

  /** Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** Preview mode of the camera. FILL_IN or FIT_IN. Default is FILL_IN. Android only */
  cameraPreviewMode?: CameraPreviewMode;

  /** Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only. */
  touchToFocusEnabled?: boolean;
}

export interface CroppingConfiguration {
  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** Text of the edit screen hint. */
  hintTitle?: string;

  /** Color of the edit screen hint text. */
  hintTitleColor?: string;

  /** Background color of the screen. */
  backgroundColor?: string;

  /** Title of the Rotate button. */
  rotateButtonTitle?: string;

  /** Title of the Reset button. */
  resetButtonTitle?: string;

  /** Title of the Detect button. */
  detectButtonTitle?: string;

  /** Title of the Done button. */
  doneButtonTitle?: string;

  /** Changes the visibility of the Rotate button. */
  rotateButtonHidden?: boolean;

  /** Changes the visibility of Detect and Reset buttons. */
  detectResetButtonHidden?: boolean;

  /** UI Interface orientation lock mode */
  orientationLockMode?: OrientationLockMode;

  /** Default color of the cropping outline. */
  polygonColor?: string;

  /** Default color of the cropping draggable elements. */
  anchorPointsColor?: string;

  /** Outline color of magnetically snapped edges. */
  polygonColorMagnetic?: string;

  /** Width of the detected document outline. */
  polygonLineWidth?: number;

  /** The background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** The color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** The background color of the bottom shutter-bar. */
  bottomBarBackgroundColor?: string;

  /** The color of the title of all buttons in the bottom shutter-bar (Cancel button, etc.), as well as the camera permission prompt button. */
  bottomBarButtonsColor?: string;

  /** Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** Swap top and bottom buttons on the screen */
  swapTopBottomButtons?: boolean;

  /** The title of the Cropping UI Screen. iOS only */
  topBarTitle?: string;

  /** The color of the title. iOS only */
  topBarTitleColor?: string;

  /** Allows you to customize the accessibility configuration for the Cropping UI */
  accessibilityConfiguration?: CroppingAccessibilityConfiguration;
}

export interface DocumentDataExtractorScreenConfiguration {
  /** Controls whether the flash should be initially enabled. The default value is FALSE. */
  flashEnabled?: boolean;

  /** UI Interface orientation lock mode */
  orientationLockMode?: OrientationLockMode;

  /** The preferred camera module (default - BACK) */
  cameraModule?: CameraModule;

  /** The background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** The color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** The color of all inactive toggle buttons in the toolbar. */
  topBarButtonsInactiveColor?: string;

  /** Background color outside of the finder window. */
  cameraOverlayColor?: string;

  /** Foreground color of the detection overlay. */
  finderLineColor?: string;

  /** Width of finder frame border. Default is 2. */
  finderLineWidth?: number;

  /** Text color of the fields count label. */
  fieldsCountTextColor?: string;

  /** Color of confidence value label background in details screen, when the field confidence level is high. */
  fieldConfidenceHighColor?: string;

  /** Color of confidence value label background in details screen, when the field confidence level is moderate. */
  fieldConfidenceModerateColor?: string;

  /** Color of confidence value label background in details screen, when the field confidence level is low. */
  fieldConfidenceLowColor?: string;

  /** Color of confidence value label text in details. */
  fieldConfidenceTextColor?: string;

  /** Color of tip text on scanning screen. */
  tipTextColor?: string;

  /** Color of tip background on scanning screen. */
  tipBackgroundColor?: string;

  /** The color of bottom sheet */
  detailsBackgroundColor?: string;

  /** The color of text elements in bottom sheet */
  detailsPrimaryColor?: string;

  /** The color of Submit button */
  detailsActionColor?: string;

  /** Text color for section headers on the details screen. iOS only. */
  detailsSectionHeaderTextColor?: string;

  /** Background color for section headers on the details screen. iOS only. */
  detailsSectionHeaderBackgroundColor?: string;

  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** Whether the cancel button is hidden or not. iOS only. */
  cancelButtonHidden?: boolean;

  /** String used for displaying amount of detected fields. Use %d for number formatting symbol. */
  clearButtonTitle?: string;

  /** Text of the button which finishes the flow */
  submitButtonTitle?: string;

  /** String used for displaying amount of detected fields. Use %d for number formatting symbol. */
  fieldsCountText?: string;

  /** String that shows average confidence value of scanned document. Use %d as number formatting symbol. */
  confidenceValue?: string;

  /** String that asks user to scan back side of a document. */
  scanBackSideTitle?: string;

  /** String that asks user to scan front side of a document. */
  scanFrontSideTitle?: string;

  /** String that asks user to start scanning a document. */
  startScanningTitle?: string;

  /** String that notifies that both sides of document are scanned. */
  scannedEverythingTitle?: string;

  /** String being displayed for empty values. iOS only. */
  emptyValueTitle?: string;

  /** Title of the button that opens the screen where the user can allow the usage of the camera by the app. */
  enableCameraButtonTitle?: string;

  /** Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera. */
  enableCameraExplanationText?: string;

  /** A title to show image content. Android only. */
  imageTitle?: string;

  /** String that notifies that nothing was scanned yet. */
  noDataTitle?: string;

  /** Accepted document types. All other document types will be ignored. By default - All types */
  acceptedDocumentTypes?: DocumentDataType[];

  /** Allows to configure the display configuration for fields. */
  fieldsDisplayConfiguration?: FieldsDisplayConfiguration[];

  /** Allows to configure the display configuration for documents. */
  documentsDisplayConfiguration?: DocumentsDisplayConfiguration[];

  /** List of secure fields which should be excluded from scanning process. All other fields will be scanned as usual. Field should be set ONLY as normalized field name. Example - [DePassport.BirthDate] or [DePassport.Birthplace] */
  excludedFieldTypes?: string[];

  /** Controls whether the flash toggle button is hidden or not. */
  flashButtonHidden?: boolean;

  /** Title of the flash toggle button. */
  flashButtonTitle?: string;

  /** Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** If `true`, replaces the cancel button in the top bar with a back arrow icon. The default value is FALSE. Android only. */
  replaceCancelButtonWithIcon?: boolean;

  /** Preview mode of the camera. FILL_IN or FIT_IN. Default is FILL_IN. Android only */
  cameraPreviewMode?: CameraPreviewMode;

  /** Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only. */
  touchToFocusEnabled?: boolean;
}

export interface DocumentScannerScreenConfiguration {
  /** The minimum score in percent (0 - 100) of the perspective distortion to accept a detected document. Default is 75.0. */
  acceptedAngleScore?: number;

  /** The minimum document width or height in percent (0 - 100) of the screen size to accept a detected document. Default is 80.0. */
  acceptedSizeScore?: number;

  /** The brightness threshold to accept a detected document in the range of 0 (very dark) to 255 (very bright). If  the average brightness value of the detectors input image is below this value, the status of the detection will be set to ERROR_TOO_DARK. */
  acceptedBrightnessThreshold?: number;

  /** Controls whether the auto-snapping toggle button is hidden or not. */
  autoSnappingButtonHidden?: boolean;

  /** Title of the auto-snapping toggle button. */
  autoSnappingButtonTitle?: string;

  /** When auto-snapping is enabled the document scanner will take a photo automatically when a document is detected, conditions are good and the auto-snapping time-out elapses. In this mode the user can still tap the shutter button to snap a document. */
  autoSnappingEnabled?: boolean;

  /** Controls the auto-snapping speed. Sensitivity must be within the 0..1 range. A value of 1.0 triggers automatic capturing immediately, a value of 0.0 delays the automatic by 3 seconds. The default value is 0.66 (2 seconds) */
  autoSnappingSensitivity?: number;

  /** The minimum delay in seconds between two consecutive automatic image captures. iOS only. */
  autoSnappingDelay?: number;

  /** The background color of the bottom shutter-bar. */
  bottomBarBackgroundColor?: string;

  /** The color of the title of all buttons in the bottom shutter-bar (Cancel button, etc.), as well as the camera permission prompt button. */
  bottomBarButtonsColor?: string;

  /** The preferred camera module (default - BACK) */
  cameraModule?: CameraModule;

  /** The color of the camera background (relevant only when the camera preview mode is CameraPreviewMode.FIT_IN). */
  cameraBackgroundColor?: string;

  /** Preview mode of the camera. Fit-In or Fill-In. */
  cameraPreviewMode?: CameraPreviewMode;

  /** Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only. */
  touchToFocusEnabled?: boolean;

  /** String being displayed on the label describing that the app is in split mode and needs to go fullscreen to work with camera. iOS only. */
  cameraUnavailableExplanationText?: string;

  /** Whether the cancel button is hidden or not. iOS only. */
  cancelButtonHidden?: boolean;

  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** The default image filter type for new documents pages. Defaults to ImageFilterTypeNone. */
  defaultPageFilter?: ImageFilterType;

  /** The mode of the document scanner to be used for document outline detection. Defaults to ML. iOS only. For Android, use the Scanbot SDK initializer. */
  engineMode?: DocumentScannerEngineMode;

  /** Title of the button that opens the screen where the user can allow the usage of the camera by the app. */
  enableCameraButtonTitle?: string;

  /** Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera. */
  enableCameraExplanationText?: string;

  /** Controls whether the flash toggle button is hidden or not. */
  flashButtonHidden?: boolean;

  /** Title of the flash toggle button. */
  flashButtonTitle?: string;

  /** Controls whether the flash should be initially enabled. The default value is FALSE. */
  flashEnabled?: boolean;

  /** Shows the user guidance elements if autosnapping is disabled. */
  forceUserGuidance?: boolean;

  /** Sets whether to ignore the OK_BUT_ORIENTATION_MISMATCH detection status. By default OrientationMismatch is not ignored. */
  ignoreOrientationMismatch?: boolean;

  /** The image scaling factor. The factor must be within the 0..1 range. A factor of 1 means that the resulting images retain their original size. When the factor is less than 1, resulting images will be made smaller by that factor. By default the scale is 1. */
  imageScale?: number;

  /** Controls whether the multi-page toggle button is hidden or not. */
  multiPageButtonHidden?: boolean;

  /** Title of the multi-page mode toggle button. */
  multiPageButtonTitle?: string;

  /** Controls multi-page mode. When enabled, the user can take multiple document photos before closing the screen by tapping the page counter button. When disabled, the screen will be closed immediately after the first document photo is made. The default value is FALSE. */
  multiPageEnabled?: boolean;

  /** UI Interface orientation lock mode */
  orientationLockMode?: OrientationLockMode;

  /** Title suffix of the button that finishes the document scanning when multi-page scanning is enabled. The button's title has the format "# Pages", where '#' shows the number of images captured up to now and the suffix "Pages" is set using this method. */
  pageCounterButtonTitle?: string;

  /** The prioritization of still image quality and capturing speed. Defaults to CapturePhotoQualityPrioritization.BALANCED. If you experience lots of blurry still images try to set this property to CapturePhotoQualityPrioritization.QUALITY. Note: Has no effect on devices prior to iOS 13.0!. iOS only. */
  photoQualityPrioritization?: CapturePhotoQualityPrioritization;

  /** The background color of the detected document outline when the document's angle, size or aspect ratio is not yet sufficiently good. (All net.doo.snap.lib.detector.DetectionResult with OK_BUT_XXX). */
  polygonBackgroundColor?: string;

  /** The background color of the detected document outline when we are ready to snap OK. */
  polygonBackgroundColorOK?: string;

  /** The color of the detected document outline when the document's angle, size or aspect ratio is not yet sufficiently good. (All detection statuses in net.doo.snap.lib.detector.DetectionResult that have the OK_BUT_XXX prefix). */
  polygonColor?: string;

  /** The color of the detected document outline when we are ready to snap OK. */
  polygonColorOK?: string;

  /** Width of the detected document outline. */
  polygonLineWidth?: number;

  /** The radius to use when drawing rounded corners of the polygon. Default is 8.0. */
  polygonCornerRadius?: number;

  /** Stroke color of polygon auto snap progress animation. Default is green. Can't be nil. */
  polygonAutoSnapProgressColor?: string;

  /** Line width of polygon auto snap progress animation. Default is 5.0. */
  polygonAutoSnapProgressLineWidth?: number;

  /** Whether polygon auto snap progress animation is enabled or not. Default is true. */
  polygonAutoSnapProgressEnabled?: boolean;

  /** An optional array of aspect ratios the documents have to conform to. */
  requiredAspectRatios?: AspectRatio[];

  /** An optional array of aspect ratios the documents may conform to. */
  preferredAspectRatios?: AspectRatio[];

  /** Whether polygon auto snap progress animation is enabled or not. Default is true. */
  shutterButtonAutoInnerColor?: string;

  /** The background color of the shutter button in auto-snapping mode. */
  shutterButtonAutoOuterColor?: string;

  /** The indicator color of the shutter button in auto-snapping mode. iOS only. */
  shutterButtonIndicatorColor?: string;

  /** The foreground color of the shutter button in manual mode. */
  shutterButtonManualInnerColor?: string;

  /** The background color of the shutter button in manual mode. */
  shutterButtonManualOuterColor?: string;

  /** The camera session will be stopped entirely, when the receiver disappears and restarts when the receiver reappears if set to True else will pause the delivery of video frames and QR Codes. Defaults to True. iOS only. */
  stopsCameraSessionWhenDisappeared?: boolean;

  /** Text hint that will be shown when the current detection status is OK_BUT_BAD_ANGLES */
  textHintBadAngles?: string;

  /** Text hint that will be shown when the current detection status is OK_BUT_BAD_ASPECT_RATIO */
  textHintBadAspectRatio?: string;

  /** Text hint that will be shown when the current detection status is ERROR_NOTHING_DETECTED */
  textHintNothingDetected?: string;

  /** The text being displayed on the user-guidance label, when a document was detected, but its center is too far away from the image center. */
  textHintOffCenter?: string;

  /** Text hint that will be shown when the current detection status is OK */
  textHintOK?: string;

  /** Text hint that will be shown when the current detection status is ERROR_TOO_DARK */
  textHintTooDark?: string;

  /** Text hint that will be shown when the current detection status is ERROR_TOO_NOISY */
  textHintTooNoisy?: string;

  /** Text hint that will be shown when the current detection status is OK_BUT_TOO_SMALL */
  textHintTooSmall?: string;

  /** The background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** The color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** The color of all inactive toggle buttons in the toolbar. */
  topBarButtonsInactiveColor?: string;

  /** The background color of the user guidance hints. */
  userGuidanceBackgroundColor?: string;

  /** The text color of the user guidance hints. */
  userGuidanceTextColor?: string;

  /** Font size of the user guidance. Default is 17.0. */
  userGuidanceFontSize?: number;

  /** Limits the maximum size of the document image. If width or height are zero, this property is effectively ignored. */
  documentImageSizeLimit?: Size;

  /** Hides the shutter button if set to TRUE. Shows it otherwise. Defaults to FALSE. If set to TRUE, auto-snapping is enabled and the property autoSnappingEnabled of the behaviour configuration will have no effect. Also the auto-snapping button will be hidden. */
  shutterButtonHidden?: boolean;

  /** The text being displayed on the user-guidance label, when the scanners energy saver is activated. iOS only. */
  textHintEnergySavingActive?: string;

  /** Maximum number of pages to scan. Ignored when set to null, or when `multiPageEnabled` is FALSE. Default value is null. */
  maxNumberOfPages?: number;

  /** Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** Allows you to customize the accessibility configuration for the Document Scanner UI */
  accessibilityConfiguration?: DocumentScannerAccessibilityConfiguration;
}

export interface HealthInsuranceCardScannerConfiguration {
  /** The preferred camera module (default - BACK) */
  cameraModule?: CameraModule;

  /** Background color outside of the finder window. */
  cameraOverlayColor?: string;

  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** Whether the cancel button is hidden or not. iOS only. */
  cancelButtonHidden?: boolean;

  /** String being displayed when health insurance card was not found. */
  detectionStatusFailedDetectionText?: string;

  /** String being displayed when health insurance card was found but not recognized. */
  detectionStatusFailedValidationText?: string;

  /** String being displayed when health insurance card was found and recognized. */
  detectionStatusSuccessText?: string;

  /** Title of the button that opens the screen where the user can allow the usage of the camera by the app. */
  enableCameraButtonTitle?: string;

  /** Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera. */
  enableCameraExplanationText?: string;

  /** Foreground color of the detection overlay. */
  finderLineColor?: string;

  /** Width of finder frame border. Default is 2. */
  finderLineWidth?: number;

  /** String being displayed as description. */
  finderTextHint?: string;

  /** Foreground color of the description label. */
  finderTextHintColor?: string;

  /** Controls whether the flash toggle button is hidden or not. iOS only. */
  flashButtonHidden?: boolean;

  /** Title of the flash toggle button. iOS only. */
  flashButtonTitle?: string;

  /** Controls whether the flash should be initially enabled. The default value is FALSE. */
  flashEnabled?: boolean;

  /** UI Interface orientation lock mode */
  orientationLockMode?: OrientationLockMode;

  /** The background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** The color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** The color of all inactive toggle buttons in the toolbar. */
  topBarButtonsInactiveColor?: string;

  /** Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** If `true`, replaces the cancel button in the top bar with a back arrow icon. The default value is FALSE. Android only. */
  replaceCancelButtonWithIcon?: boolean;

  /** Preview mode of the camera. FILL_IN or FIT_IN. Default is FILL_IN. Android only */
  cameraPreviewMode?: CameraPreviewMode;

  /** Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only. */
  touchToFocusEnabled?: boolean;

  /** The live detection configuration parameters. */
  recognizerParameters?: HealthInsuranceCardRecognizerParameters;
}

export interface FinderDocumentScannerConfiguration {
  /** Minimum score in percent (0 - 100) of the accepted perspective distortion of the detected document. Default is 75.0. */
  acceptedAngleScore?: number;

  /** Minimum score in percent (0 - 100) of accepted document width or height to screen size ratio of the detected document. Default is 80.0. */
  acceptedSizeScore?: number;

  /** The brightness threshold to accept a detected document in the range of 0 (very dark) to 255 (very bright). If  the average brightness value of the detectors input image is below this value, the status of the detection will be set to ERROR_TOO_DARK. */
  acceptedBrightnessThreshold?: number;

  /** Whether to enable auto-snapping. If enabled, the document scanner will take a photo automatically when a document is detected, conditions are good and the auto-snapping time-out elapses. In this mode, the user can still tap the shutter button to snap a document. */
  autoSnappingEnabled?: boolean;

  /** Controls the auto-snapping speed. Sensitivity must be within the 0..1 range. A value of 1.0 triggers automatic capturing immediately, a value of 0.0 delays the automatic capturing by 3 seconds. Default value is 0.66 (2 seconds). */
  autoSnappingSensitivity?: number;

  /** Minimum delay in seconds between two consecutive automatic image captures. iOS only. */
  autoSnappingDelay?: number;

  /** Preferred camera module (Default: BACK). */
  cameraModule?: CameraModule;

  /** Color of the camera background (relevant only when the camera preview mode is CameraPreviewMode.FIT_IN). */
  cameraBackgroundColor?: string;

  /** Preview mode of the camera. Fit-In or Fill-In. */
  cameraPreviewMode?: CameraPreviewMode;

  /** Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only. */
  touchToFocusEnabled?: boolean;

  /** String being displayed on the label describing that the app is in split mode and needs to go fullscreen to work with the camera. iOS only. */
  cameraUnavailableExplanationText?: string;

  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** Default image filter type for new document pages. Defaults to ImageFilterTypeNone. */
  defaultPageFilter?: ImageFilterType;

  /** Document scanner mode to be used for detecting the document outline. Defaults to ML. iOS only. For Android, use the Scanbot SDK initializer. */
  engineMode?: DocumentScannerEngineMode;

  /** Title of the button that opens the screen where the user can allow the usage of the camera by the app. */
  enableCameraButtonTitle?: string;

  /** Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera. */
  enableCameraExplanationText?: string;

  /** Whether the flash toggle button is hidden. */
  flashButtonHidden?: boolean;

  /** Whether the flash should be initially enabled. The default value is FALSE. */
  flashEnabled?: boolean;

  /** Whether to show the user guidance elements if auto-snapping is disabled. */
  forceUserGuidance?: boolean;

  /** Whether to enable the view finder. */
  finderEnabled?: boolean;

  /** Foreground color of the detection overlay. */
  finderLineColor?: string;

  /** Width of finder frame border. Default is 2. */
  finderLineWidth?: number;

  /** Background color outside of the finder window. */
  cameraOverlayColor?: string;

  /** Aspect ratio of the finder frame (width to height), which is used to build the actual finder frame. Default is the DIN A4 aspect ratio. */
  finderAspectRatio?: AspectRatio;

  /** Sets whether to ignore the OK_BUT_ORIENTATION_MISMATCH detection status. By default OrientationMismatch is not ignored. */
  ignoreOrientationMismatch?: boolean;

  /** Image scaling factor. The factor must be within the 0..1 range. A factor of 1 means that the resulting images retain their original size. When the factor is less than 1, resulting images will be made smaller by that factor. By default the scale is 1. */
  imageScale?: number;

  /** Mode for locking the UI's orientation. */
  orientationLockMode?: OrientationLockMode;

  /** Prioritization of still image quality and capturing speed. Defaults to CapturePhotoQualityPrioritization.BALANCED. If you experience lots of blurry still images, try to set this property to CapturePhotoQualityPrioritization.QUALITY. iOS only. */
  photoQualityPrioritization?: CapturePhotoQualityPrioritization;

  /** Whether to display the document's polygon. */
  polygonEnabled?: boolean;

  /** Background color of the detected document's outline when the document's angle, size or aspect ratio is not yet sufficiently good. (All net.doo.snap.lib.detector.DetectionResult with OK_BUT_XXX.) */
  polygonBackgroundColor?: string;

  /** Background color of the detected document's outline when ready to snap. */
  polygonBackgroundColorOK?: string;

  /** Color of the detected document's outline when the document's angle, size or aspect ratio is not yet sufficiently good. (All detection statuses in net.doo.snap.lib.detector.DetectionResult that have the OK_BUT_XXX prefix.) */
  polygonColor?: string;

  /** Color of the detected document's outline when ready to snap. */
  polygonColorOK?: string;

  /** Width of the detected document's outline. */
  polygonLineWidth?: number;

  /** Radius to use when drawing rounded corners of the polygon. Default is 8.0. */
  polygonCornerRadius?: number;

  /** Stroke color of the polygon or the finder's auto-snap progress animation. Default is green. Can't be nil. */
  autoSnapProgressColor?: string;

  /** Line width of the polygon or the finder's auto-snap progress animation. Default is 5.0. */
  autoSnapProgressLineWidth?: number;

  /** Whether to enable the polygon or finder's auto-snap progress animation. Default is true. */
  autoSnapProgressEnabled?: boolean;

  /** Whether to allow scanning only for documents with the same aspect ratio as the finder. */
  lockDocumentAspectRatioToFinder?: boolean;

  /** Foreground color of the shutter button in auto-snapping mode. */
  shutterButtonAutoInnerColor?: string;

  /** Background color of the shutter button in auto-snapping mode. */
  shutterButtonAutoOuterColor?: string;

  /** Indicator color of the shutter button in auto-snapping mode. iOS only. */
  shutterButtonIndicatorColor?: string;

  /** Foreground color of the shutter button in manual mode. */
  shutterButtonManualInnerColor?: string;

  /** Background color of the shutter button in manual mode. */
  shutterButtonManualOuterColor?: string;

  /** If true, the camera session is stopped entirely when the receiver disappears and restarted when the receiver reappears. Else, will pause the delivery of video frames. Defaults to True. iOS only. */
  stopsCameraSessionWhenDisappeared?: boolean;

  /** Text hint that will be shown when the current detection status is OK_BUT_BAD_ANGLES. */
  textHintBadAngles?: string;

  /** Text hint that will be shown when the current detection status is OK_BUT_BAD_ASPECT_RATIO. */
  textHintBadAspectRatio?: string;

  /** Text hint that will be shown when the current detection status is ERROR_NOTHING_DETECTED. */
  textHintNothingDetected?: string;

  /** Text hint that will be show when a document was detected, but its center is too far away from the image center. */
  textHintOffCenter?: string;

  /** Text hint that will be shown when the current detection status is OK. */
  textHintOK?: string;

  /** Text hint that will be shown when the current detection status is ERROR_TOO_DARK. */
  textHintTooDark?: string;

  /** Text hint that will be shown when the current detection status is ERROR_TOO_NOISY. */
  textHintTooNoisy?: string;

  /** Text hint that will be shown when the current detection status is OK_BUT_TOO_SMALL. */
  textHintTooSmall?: string;

  /** Background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** Color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** Color of all inactive toggle buttons in the toolbar. */
  topBarButtonsInactiveColor?: string;

  /** Background color of the user guidance hints. */
  userGuidanceBackgroundColor?: string;

  /** Text color of the user guidance hints. */
  userGuidanceTextColor?: string;

  /** Font size of the user guidance. Default is 17.0. */
  userGuidanceFontSize?: number;

  /** Limits the maximum size of the document image. If the width or height are zero, this property is effectively ignored. */
  documentImageSizeLimit?: Size;

  /** Whether to hide the shutter button. Defaults to TRUE. If set to TRUE, auto-snapping is enabled and the property autoSnappingEnabled of the behavior configuration will have no effect. Also the auto-snapping button will be hidden. */
  shutterButtonHidden?: boolean;

  /** Text hint that will be shown when the scanner's energy saver is activated. iOS only. */
  textHintEnergySavingActive?: string;

  /** Whether buttons should use all-capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** Allows you to customize the accessibility configuration for the Document Scanner UI. */
  accessibilityConfiguration?: FinderDocumentScannerAccessibilityConfiguration;
}

export interface MedicalCertificateScannerConfiguration {
  /** The preferred camera module (default - BACK) */
  cameraModule?: CameraModule;

  /** Background color outside of the finder window. */
  cameraOverlayColor?: string;

  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** Whether the cancel button is hidden or not. iOS only. */
  cancelButtonHidden?: boolean;

  /** Title of the button that opens the screen where the user can allow the usage of the camera by the app. */
  enableCameraButtonTitle?: string;

  /** Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera. */
  enableCameraExplanationText?: string;

  /** Foreground color of the detection overlay. */
  finderLineColor?: string;

  /** Width of finder frame border. Default is 2. */
  finderLineWidth?: number;

  /** Controls whether the flash toggle button is hidden or not. iOS only. */
  flashButtonHidden?: boolean;

  /** Title of the flash toggle button. iOS only. */
  flashButtonTitle?: string;

  /** Controls whether the flash should be initially enabled. The default value is FALSE. */
  flashEnabled?: boolean;

  /** UI Interface orientation lock mode */
  orientationLockMode?: OrientationLockMode;

  /** The background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** The color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** The color of all inactive toggle buttons in the toolbar. */
  topBarButtonsInactiveColor?: string;

  /** Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** The background color of the user guidance hints. */
  userGuidanceBackgroundColor?: string;

  /** The text color of the user guidance hints. */
  userGuidanceTextColor?: string;

  /** Whether the user guidance label is hidden or not. iOS only. */
  userGuidanceHidden?: boolean;

  /** Whether sdk should run recognition of Patient Information block. */
  recognizePatientInfo?: boolean;

  /** Whether sdk should return cropped document image or not */
  returnCroppedDocumentImage?: boolean;

  /** The style of the dialog/alert in case of error. iOS only. */
  errorDialogStyle?: DialogStyle;

  /** Error dialog title */
  errorDialogTitle?: string;

  /** The accent color of buttons on a error dialog. */
  errorDialogAccentColor?: string;

  /** Error dialog message */
  errorDialogMessage?: string;

  /** Error dialog ok button text. */
  errorDialogOkButton?: string;

  /** Allows to set if the confirm button should be filled. Android only. */
  errorDialogOkButtonFilled?: boolean;

  /** Allows to set a text color of the filled button. Android only. */
  errorDialogOkButtonFilledTextColor?: string;

  /** The style of the dialogs/alerts OK button in case of error. iOS only. */
  errorDialogButtonStyle?: DialogButtonStyle;

  /** Option to show or hide finder overlay */
  showFinderOverlay?: boolean;

  /** Configuration that helps to override default hint values */
  userGuidanceStrings?: MedicalCertificateUserGuidanceStrings;

  /** Preview mode of the camera. FILL_IN or FIT_IN. Default is FILL_IN. Android only */
  cameraPreviewMode?: CameraPreviewMode;

  /** Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only. */
  touchToFocusEnabled?: boolean;

  /** Aspect ratio of the view finder (width \ height). Set to nil of no view finder should be displayed. iOS only. */
  finderAspectRatio?: AspectRatio;

  /** Secondary aspect ratio of finder frame (width \ height). Set to nil if no secondary view finder should be displayed. iOS only. */
  finderSecondaryAspectRatio?: AspectRatio;
}

export interface ScanbotSdkConfiguration {
  /** Optional boolean flag to enable logging. See the "Logging" section below. */
  loggingEnabled?: boolean;

  /** Optional boolean flag that enables Scanbot SDK Core native logging (default is false, Android only). */
  enableNativeLogging?: boolean;

  /** Your license key for the Scanbot SDK. See the "License Key" section below. */
  licenseKey?: string;

  /** Optional image quality value. It defines the quality factor of JPEG images. The value must be between 1 and 100, where 100 means maximum quality and largest file size. The recommended default value is 80 which is a good compromise between image file size and document legibility. */
  storageImageQuality?: number;

  /** Optional image format, either 'JPG' or 'PNG'. The recommended default value is 'JPG'. Please note that 'PNG' will result in larger image files! Also, the storageImageQuality value does not apply for 'PNG'. */
  storageImageFormat?: StorageImageFormat;

  /** Optional directory as file URI to override the default storage base directory of the Scanbot SDK. Refer to the section "Storage" for more details. */
  storageBaseDirectory?: string;

  /** Optional mode for document scanner engine. */
  documentScannerEngineMode?: DocumentScannerEngineMode;

  /** Optional file encryption mode, 'AES128' or 'AES256'. */
  fileEncryptionMode?: FileEncryptionMode;

  /** Optional file encryption password. Refer to the section "Storage Encryption" for more details. */
  fileEncryptionPassword?: string;

  /** If set to `true`, Camera X will be used for the RTU-UI components (Android only). Default is `true`. */
  useCameraX?: boolean;

  /** If set to `true`, GPU Acceleration will be enabled for Barcode Scanner, Document Scanner and Generic Document Recognizer (Android only). Default is `true`. */
  allowGpuAcceleration?: boolean;

  /** Enables/disables XNNPACK acceleration for TensorFlow ML models, which provides highly optimized implementations of floating-point neural network operators (Android only) */
  allowXnnpackAcceleration?: boolean;
}

export interface VinScannerConfiguration {
  /** The preferred camera module (default - BACK) */
  cameraModule?: CameraModule;

  /** Background color outside of the finder window. */
  cameraOverlayColor?: string;

  /** Title of the cancel button. */
  cancelButtonTitle?: string;

  /** Whether the cancel button is hidden or not. iOS only. */
  cancelButtonHidden?: boolean;

  /** Title of the button that opens the screen where the user can allow the usage of the camera by the app. */
  enableCameraButtonTitle?: string;

  /** Text that will be displayed when the app is not allowed to use the camera, prompting the user to enable the usage of the camera. */
  enableCameraExplanationText?: string;

  /** Foreground color of the detection overlay. */
  finderLineColor?: string;

  /** Width of finder frame border. Default is 2. */
  finderLineWidth?: number;

  /** Foreground color of the description label. */
  finderTextHintColor?: string;

  /** Controls whether the flash toggle button is hidden or not. iOS only. */
  flashButtonHidden?: boolean;

  /** Title of the flash toggle button. iOS only. */
  flashButtonTitle?: string;

  /** Controls whether the flash should be initially enabled. The default value is FALSE. */
  flashEnabled?: boolean;

  /** UI Interface orientation lock mode */
  orientationLockMode?: OrientationLockMode;

  /** The background color of the top toolbar. */
  topBarBackgroundColor?: string;

  /** The color of all active toggle buttons in the toolbar. */
  topBarButtonsActiveColor?: string;

  /** The color of all inactive toggle buttons in the toolbar. */
  topBarButtonsInactiveColor?: string;

  /** Controls whether the scanner should try to extract the VIN from possibly available barcodes. The default value is FALSE. */
  extractVINFromBarcode?: boolean;

  /** Minimum number of accumulated frames that have equal result */
  minimumNumberOfRequiredFramesWithEqualScanningResult?: number;

  /** Maximum number of accumulated frames to inspect before actual result is returned */
  maximumNumberOfAccumulatedFrames?: number;

  /** The image will be downscaled to the given value during the processing. If the quality of the scanning is not enough, try to increase the limit. This variable affects the performance of the scanning process. Higher the size limit - better the quality, but the recognition is slower. 0 - do not rescale. Default value is 0. */
  ocrResolutionLimit?: number;

  /** Controls whether buttons should use all capitals style, as defined by the Android Material Design. Defaults to TRUE. Android only. */
  useButtonsAllCaps?: boolean;

  /** If `true`, replaces the cancel button in the top bar with a back arrow icon. The default value is FALSE. Android only. */
  replaceCancelButtonWithIcon?: boolean;

  /** Preview mode of the camera. FILL_IN or FIT_IN. Default is FILL_IN. Android only */
  cameraPreviewMode?: CameraPreviewMode;

  /** Whether touch-to-focus is enabled on camera preview. Enabled by default. Android only. */
  touchToFocusEnabled?: boolean;

  /** The cameras zoom level preferred for this step. The actual zoom might be different from the preferred one to avoid clipping of finder area and maintain its aspect ratio and height */
  aspectRatio?: AspectRatio;

  /** User guidance hint text. */
  guidanceText?: string;

  /** Threshold used to pause the detection after significant movement occurred. Default = 0. Android only. */
  significantShakeDelay?: number;
}
